@@ -19,14 +19,18 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-url.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h>
+#include <libgnome/gnome-util.h>
 #include <gdk/gdkx.h>
 
+#include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-glib.h>
+#include <libpanel-util/panel-keyfile.h>
+#include <libpanel-util/panel-show.h>
+
 #include "launcher.h"
 
 #include "button-widget.h"
-#include "menu.h"
 #include "panel-util.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
@@ -39,6 +43,7 @@
 #include "panel-lockdown.h"
 #include "panel-compatibility.h"
 #include "panel-ditem-editor.h"
+#include "panel-icon-names.h"
 
 static GdkScreen *
 launcher_get_screen (Launcher *launcher)
@@ -91,6 +96,25 @@ launcher_register_error_dialog (Launcher *launcher,
 			  launcher);
 }
 
+static GFile *
+panel_launcher_get_gfile (const char *location)
+{
+	char  *path;
+	GFile *file;
+
+	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		return g_file_new_for_uri (location);
+
+	if (g_path_is_absolute (location))
+		return g_file_new_for_path (location);
+
+	path = panel_make_full_path (NULL, location);
+	file = g_file_new_for_path (path);
+	g_free (path);
+
+	return file;
+}
+
 static char *
 panel_launcher_get_uri (const char *location)
 {
@@ -105,7 +129,7 @@ panel_launcher_get_uri (const char *location)
 	else
 		path = g_strdup (location);
 
-	uri = gnome_vfs_get_uri_from_local_path (path);
+	uri = g_filename_to_uri (path, NULL, NULL);
 	g_free (path);
 
 	return uri;
@@ -131,15 +155,13 @@ static void
 launch_url (Launcher *launcher)
 {
 	char *url;
-	char *canonical_url;
-	GError *error = NULL;
 	GdkScreen *screen;
 
 	g_return_if_fail (launcher != NULL);
 	g_return_if_fail (launcher->key_file != NULL);
 
 	/* FIXME panel_ditem_launch() should be enough for this! */
-	url = panel_util_key_file_get_string (launcher->key_file, "URL");
+	url = panel_key_file_get_string (launcher->key_file, "URL");
 
 	screen = launcher_get_screen (launcher);
 
@@ -155,26 +177,9 @@ launch_url (Launcher *launcher)
 		return;
 	}
 
-	canonical_url = gnome_vfs_make_uri_canonical (url);
-	gnome_url_show_on_screen (canonical_url, screen, &error);
-
-	if (error) {
-		GtkWidget *error_dialog;
-		char      *primary;
-	
-		primary = g_strdup_printf (_("Could not show '%s'"),
-					   canonical_url);
-		error_dialog = panel_error_dialog (NULL, screen,
-						   "cannot_show_url_dialog",
-						   TRUE,
-						   primary, error->message);
-		g_free (primary);
-		launcher_register_error_dialog (launcher, error_dialog);
-		g_clear_error (&error);
-	}
+	panel_show_uri (screen, url, gtk_get_current_event_time (), NULL);
 
 	g_free (url);
-	g_free (canonical_url);
 }
 
 void
@@ -187,9 +192,12 @@ launcher_launch (Launcher  *launcher,
 	g_return_if_fail (launcher->key_file != NULL);
 
 	if (panel_global_config_get_enable_animations ())
-		xstuff_zoom_animate (widget, NULL);
+		xstuff_zoom_animate (widget,
+				     button_widget_get_pixbuf (BUTTON_WIDGET (widget)),
+				     button_widget_get_orientation (BUTTON_WIDGET (widget)),
+				     NULL);
 	
-	type = panel_util_key_file_get_string (launcher->key_file, "Type");
+	type = panel_key_file_get_string (launcher->key_file, "Type");
 	if (type && !strcmp (type, "Link"))
 		launch_url (launcher);
 	else {
@@ -244,27 +252,30 @@ drag_data_received_cb (GtkWidget        *widget,
 		       guint             time,
 		       Launcher         *launcher)
 {
-	GError     *error = NULL;
-	GList      *li;
-	GList      *list;
+	GError  *error = NULL;
+	char   **uris;
+	int      i;
+	GList   *file_list;
 
 	if (panel_global_config_get_enable_animations ())
-		xstuff_zoom_animate (widget, NULL);
+		xstuff_zoom_animate (widget,
+				     button_widget_get_pixbuf (BUTTON_WIDGET (widget)),
+				     button_widget_get_orientation (BUTTON_WIDGET (widget)),
+				     NULL);
 	
-	list = gnome_vfs_uri_list_parse ((const char*) selection_data->data);
-	for (li = list; li != NULL; li = li->next) {
-		GnomeVFSURI *uri = li->data;
-		li->data = gnome_vfs_uri_to_string (uri, 0 /* hide_options */);
-		gnome_vfs_uri_unref (uri);
-	}
+	file_list = NULL;
+	uris = g_uri_list_extract_uris ((const char *) selection_data->data);
+	for (i = 0; uris[i]; i++)
+		file_list = g_list_prepend (file_list, uris[i]);
+	file_list = g_list_reverse (file_list);
 
 	panel_util_launch_from_key_file (launcher->key_file,
-					 list,
+					 file_list,
 					 launcher_get_screen (launcher),
 					 &error);
 
-	g_list_foreach (list, (GFunc)g_free, NULL);
-	g_list_free (list);
+	g_list_free (file_list);
+	g_strfreev (uris);
 
 	if (error) {
 		GtkWidget *error_dialog;
@@ -320,29 +331,37 @@ free_launcher (gpointer data)
 void
 panel_launcher_delete (Launcher *launcher)
 {
-	char *location;
-	char *p;
+	GFile *file;
+	GFile *launchers;
+	char  *launchers_path;
 
 	if (!launcher->location)
 		return;
 
-	p = NULL;
+	launchers_path = gnome_util_home_file (PANEL_LAUNCHERS_PATH);
+	launchers = g_file_new_for_path (launchers_path);
+	g_free (launchers_path);
 
-	location = panel_launcher_get_uri (launcher->location);
-	if (location)
-		p = strstr (location, PANEL_LAUNCHERS_PATH);
+	file = panel_launcher_get_gfile (launcher->location);
 
 	/* do not remove the file if it's not in the user's launchers path */
-	if (p) {
-		GnomeVFSResult result;
+	if (g_file_has_prefix (file, launchers)) {
+		GError *error;
 
-		result = gnome_vfs_unlink (location);
-		if (result != GNOME_VFS_OK)
-			g_warning ("Error unlinking '%s': %s\n", location,
-				   gnome_vfs_result_to_string (result));
+		error = NULL;
+		if (!g_file_delete (file, NULL, &error)) {
+			char *path;
+
+			path = g_file_get_path (file);
+			g_warning ("Error deleting '%s': %s\n",
+				   path, error->message);
+			g_free (path);
+			g_error_free (error);
+		}
 	}
 
-	g_free (location);
+	g_object_unref (file);
+	g_object_unref (launchers);
 }
 
 static gboolean
@@ -362,7 +381,7 @@ is_this_drop_ok (GtkWidget      *widget,
 		return FALSE;
 
 	if (!text_uri_list)
-		text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
+		text_uri_list = gdk_atom_intern_static_string ("text/uri-list");
 
 	for (l = context->targets; l; l = l->next) {
 		if (GDK_POINTER_TO_ATOM (l->data) == text_uri_list)
@@ -413,8 +432,8 @@ drag_drop_cb (GtkWidget	        *widget,
 	if ( ! is_this_drop_ok (widget, context))
 		return FALSE;
 
-	if (text_uri_list == 0)
-		text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
+	if (text_uri_list == NULL)
+		text_uri_list = gdk_atom_intern_static_string ("text/uri-list");
 
 	gtk_drag_get_data (widget, context, text_uri_list, time);
 
@@ -483,12 +502,8 @@ create_launcher (const char *location)
 		path = panel_make_full_path (NULL, location);
 
 		if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
-			char *buffer;
-
 			g_free (path);
-			buffer = g_strconcat ("applications/", location, NULL);
-			path = panel_lookup_in_data_dirs (buffer);
-			g_free (buffer);
+			path = panel_g_lookup_in_applications_dirs (location);
 			/* it's important to keep the full path if the desktop
 			 * file comes from a data dir: when the user will edit
 			 * it, we'll want to save it in PANEL_LAUNCHERS_PATH
@@ -506,9 +521,9 @@ create_launcher (const char *location)
 			g_free (path);
 		}
 	} else
-		loaded = panel_util_key_file_load_from_uri (key_file, location,
-							    G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS,
-							    &error);
+		loaded = panel_key_file_load_from_uri (key_file, location,
+						       G_KEY_FILE_KEEP_COMMENTS|G_KEY_FILE_KEEP_TRANSLATIONS,
+						       &error);
 
 	if (!loaded) {
 		g_printerr (_("Unable to open desktop file %s for panel launcher%s%s\n"),
@@ -570,30 +585,6 @@ create_launcher (const char *location)
 	return launcher;
 }
 
-static char *
-guess_icon_from_exec (GtkIconTheme *icon_theme,
-		      GKeyFile     *key_file)
-{
-	char *exec;
-	char *icon_name;
-	char *retval;
-
-	exec = panel_util_key_file_get_string (key_file, "Exec");
-
-	if (!exec || !exec [0]) {
-		g_free (exec);
-		return NULL;
-	}
-
-	icon_name = g_path_get_basename (exec);
-	retval = panel_find_icon (icon_theme, icon_name, 48);
-
-	g_free (exec);
-	g_free (icon_name);
-
-	return retval;
-}
-
 static void
 setup_button (Launcher *launcher)
 {
@@ -605,16 +596,17 @@ setup_button (Launcher *launcher)
 	
 	g_return_if_fail (launcher != NULL);
 
-	name = panel_util_key_file_get_locale_string (launcher->key_file,
-						      "Name");
-	comment = panel_util_key_file_get_locale_string (launcher->key_file,
-							 "Comment");
+	name = panel_key_file_get_locale_string (launcher->key_file, "Name");
+	comment = panel_key_file_get_locale_string (launcher->key_file,
+						    "Comment");
 
 	/* Setup tooltip */
-	if ( ! string_empty (comment))
+	if (!PANEL_GLIB_STR_EMPTY (name) && !PANEL_GLIB_STR_EMPTY (comment))
 		str = g_strdup_printf ("%s\n%s", name, comment);
-	else
+	else if (!PANEL_GLIB_STR_EMPTY (name))
 		str = g_strdup (name);
+	else
+		str = g_strdup (comment);
 
 	g_free (name);
 	g_free (comment);
@@ -623,54 +615,64 @@ setup_button (Launcher *launcher)
 	 * string (a location e.g.). If we can't, then it most probably means
 	 * we have a % that is not here to encode a character, and we don't
 	 * want to unescape in this case. See bug #170516 for details. */
-	unescaped_str = gnome_vfs_unescape_string (str, NULL);
+	unescaped_str = g_uri_unescape_string (str, NULL);
 	if (unescaped_str) {
 		g_free (str);
 		str = unescaped_str;
 	}
 
-	gtk_tooltips_set_tip (panel_tooltips, launcher->button, str, NULL);
+	panel_util_set_tooltip_text (launcher->button, str);
+
 	/* Setup accessible name */
 	panel_a11y_set_atk_name_desc (launcher->button, str, NULL);
 
 	g_free (str);
 
 	/* Setup icon */
-	icon = panel_util_key_file_get_locale_string (launcher->key_file,
-						      "Icon");
-	if (!icon)
-		icon = guess_icon_from_exec (BUTTON_WIDGET (launcher->button)->icon_theme,
-					     launcher->key_file);
+	icon = panel_key_file_get_locale_string (launcher->key_file, "Icon");
+	if (icon && icon[0] == '\0') {
+		g_free (icon);
+		icon = NULL;
+	}
 
-	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), sure_string (icon));
+	if (!icon)
+		icon = guess_icon_from_exec (button_widget_get_icon_theme (BUTTON_WIDGET (launcher->button)),
+					     launcher->key_file);
+	if (!icon)
+		icon = g_strdup (PANEL_ICON_LAUNCHER);
+
+	button_widget_set_icon_name (BUTTON_WIDGET (launcher->button), icon);
 	g_free (icon);
 }
 
 static char *
-panel_launcher_find_writable_uri (const char *launcher_location)
+panel_launcher_find_writable_uri (const char *launcher_location,
+				  const char *source)
 {
 	char *path;
 	char *uri;
 
 	if (!launcher_location)
-		return panel_make_unique_uri (NULL, ".desktop");;
+		return panel_make_unique_desktop_uri (NULL, source);
 
 	if (!strchr (launcher_location, G_DIR_SEPARATOR)) {
 		path = panel_make_full_path (NULL, launcher_location);
-		uri = gnome_vfs_get_uri_from_local_path (path);
+		uri = g_filename_to_uri (path, NULL, NULL);
 		g_free (path);
 		return uri;
 	}
 
 	if (panel_launcher_get_filename (launcher_location) != NULL) {
-		if (g_path_is_absolute (launcher_location)) {
-			uri = gnome_vfs_get_uri_from_local_path (launcher_location);
-			return uri;
-		} else
+		/* we have a file in the user directory. We either have a path
+		 * or an URI */
+		if (g_path_is_absolute (launcher_location))
+			return g_filename_to_uri (launcher_location,
+						  NULL, NULL);
+		else
 			return g_strdup (launcher_location);
 	}
 
-	return panel_make_unique_uri (NULL, ".desktop");;
+	return panel_make_unique_desktop_uri (NULL, source);
 }
 
 static void
@@ -693,14 +695,12 @@ launcher_command_changed (PanelDItemEditor *dialog,
 	revert_key_file = panel_ditem_editor_get_revert_key_file (dialog);
 
 	if (revert_key_file) {
-		exec = panel_util_key_file_get_string (launcher->key_file,
-						       "Exec");
-		old_exec = panel_util_key_file_get_string (revert_key_file,
-							   "Exec");
+		exec = panel_key_file_get_string (launcher->key_file, "Exec");
+		old_exec = panel_key_file_get_string (revert_key_file, "Exec");
 
 		if (!old_exec || !exec || strcmp (old_exec, exec))
-			panel_util_key_file_remove_key (launcher->key_file,
-							"StartupNotify");
+			panel_key_file_remove_key (launcher->key_file,
+						   "StartupNotify");
 
 		g_free (exec);
 		g_free (old_exec);
@@ -711,16 +711,32 @@ static char *
 launcher_save_uri (PanelDItemEditor *dialog,
 		   gpointer          data)
 {
+	GKeyFile   *key_file;
+	char       *type;
+	char       *exec_or_uri;
 	Launcher   *launcher;
 	char       *new_uri;
 	const char *uri;
 
+	key_file = panel_ditem_editor_get_key_file (dialog);
+	type = panel_key_file_get_string (key_file, "Type");
+	if (type && !strcmp (type, "Application"))
+		exec_or_uri = panel_key_file_get_string (key_file, "Exec");
+	else if (type && !strcmp (type, "Link"))
+		exec_or_uri = panel_key_file_get_string (key_file, "URL");
+	else
+		exec_or_uri = panel_key_file_get_string (key_file, "Name");
+	g_free (type);
+
 	launcher = (Launcher *) data;
 
 	if (launcher)
-		new_uri = panel_launcher_find_writable_uri (launcher->location);
+		new_uri = panel_launcher_find_writable_uri (launcher->location,
+							    exec_or_uri);
 	else
-		new_uri = panel_launcher_find_writable_uri (NULL);
+		new_uri = panel_launcher_find_writable_uri (NULL, exec_or_uri);
+
+	g_free (exec_or_uri);
 
 	uri = panel_ditem_editor_get_uri (dialog);
 
@@ -780,7 +796,7 @@ launcher_properties (Launcher  *launcher)
 		return;
 	}
 
-	launcher->prop_dialog = panel_ditem_editor_new (GTK_WINDOW (PANEL_WIDGET (launcher->info->widget->parent)->toplevel),
+	launcher->prop_dialog = panel_ditem_editor_new (NULL,
 							launcher->key_file,
 							launcher->location,
 							_("Launcher Properties"));
@@ -950,8 +966,8 @@ ask_about_launcher (const char  *file,
 
 	key_file = panel_ditem_editor_get_key_file (PANEL_DITEM_EDITOR (dialog));
 	if (file != NULL)
-		panel_util_key_file_set_string (key_file, "Exec", file);
-	panel_util_key_file_set_string (key_file, "Type", "Application");
+		panel_key_file_set_string (key_file, "Exec", file);
+	panel_key_file_set_string (key_file, "Type", "Application");
 	panel_ditem_editor_sync_display (PANEL_DITEM_EDITOR (dialog));
 
 	panel_ditem_register_save_uri_func (PANEL_DITEM_EDITOR (dialog),
@@ -986,30 +1002,30 @@ panel_launcher_create_from_info (PanelToplevel *toplevel,
 	char     *location;
 	GError   *error;
 
-	key_file = panel_util_key_file_new_desktop ();
+	key_file = panel_key_file_new_desktop ();
 
 	/* set current language and the "C" locale to this name,
 	 * this is kind of evil... */
-	panel_util_key_file_set_string (key_file, "Name", name);
-	panel_util_key_file_set_string (key_file, "Comment", comment);
-	panel_util_key_file_set_string (key_file, "Icon", icon);
-	panel_util_key_file_set_locale_string (key_file, "Name", name);
-	panel_util_key_file_set_locale_string (key_file, "Comment", comment);
-	panel_util_key_file_set_locale_string (key_file, "Icon", icon);
+	panel_key_file_set_string (key_file, "Name", name);
+	panel_key_file_set_string (key_file, "Comment", comment);
+	panel_key_file_set_string (key_file, "Icon", icon);
+	panel_key_file_set_locale_string (key_file, "Name", name);
+	panel_key_file_set_locale_string (key_file, "Comment", comment);
+	panel_key_file_set_locale_string (key_file, "Icon", icon);
 
 	if (exec_info) {
-		panel_util_key_file_set_string (key_file, "Exec", exec_or_uri);
-		panel_util_key_file_set_string (key_file, "Type", "Application");
+		panel_key_file_set_string (key_file, "Exec", exec_or_uri);
+		panel_key_file_set_string (key_file, "Type", "Application");
 	} else {
-		panel_util_key_file_set_string (key_file, "URL", exec_or_uri);
-		panel_util_key_file_set_string (key_file, "Type", "Link");
+		panel_key_file_set_string (key_file, "URL", exec_or_uri);
+		panel_key_file_set_string (key_file, "Type", "Link");
 	}
 
-	location = panel_make_unique_uri (NULL, ".desktop");
-	if (panel_util_key_file_to_file (key_file, location, &error)) {
+	location = panel_make_unique_desktop_uri (NULL, exec_or_uri);
+	if (panel_key_file_to_file (key_file, location, &error)) {
 		panel_launcher_create (toplevel, position, location);
 	} else {
-		panel_error_dialog (NULL,
+		panel_error_dialog (GTK_WINDOW (toplevel),
 				    gtk_window_get_screen (GTK_WINDOW (toplevel)),
 				    "cannot_save_launcher", TRUE,
 				    _("Could not save launcher"),
@@ -1028,6 +1044,7 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 	GConfClient *client;
 	const char  *key;
 	char        *id;
+	char        *no_uri;
 	const char  *new_location;
 
 	g_return_if_fail (location != NULL);
@@ -1039,9 +1056,17 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 						   position,
 						   FALSE);
 
-	new_location = panel_launcher_get_filename (location);
+	no_uri = NULL;
+	/* if we have an URI, it might contain escaped characters (? : etc)
+	 * that might get unescaped on disk */
+	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
+		no_uri = g_filename_from_uri (location, NULL, NULL);
+	if (!no_uri)
+		no_uri = g_strdup (location);
+
+	new_location = panel_launcher_get_filename (no_uri);
 	if (new_location == NULL)
-		new_location = location;
+		new_location = no_uri;
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS,
 				    id,
@@ -1050,6 +1075,7 @@ panel_launcher_create_with_id (const char    *toplevel_id,
 
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
 
+	g_free (no_uri);
 	g_free (id);
 }
 
@@ -1068,33 +1094,21 @@ panel_launcher_create_copy (PanelToplevel *toplevel,
 			    int            position,
 			    const char    *location)
 {
-	GnomeVFSResult  vfs_result;
-	GnomeVFSURI    *source_uri;
-	GnomeVFSURI    *dest_uri;
-	char           *old_location;
-	char           *new_location;
-	const char     *filename;
+	char       *new_location;
+	GFile      *source;
+	GFile      *dest;
+	gboolean    copied;
+	const char *filename;
 
-	new_location = panel_make_unique_uri (NULL, ".desktop");
+	new_location = panel_make_unique_desktop_uri (NULL, location);
 
-	old_location = panel_launcher_get_uri (location);
+	source = panel_launcher_get_gfile (location);
+	dest = g_file_new_for_uri (new_location);
 
-	source_uri = gnome_vfs_uri_new (old_location);
-	dest_uri   = gnome_vfs_uri_new (new_location);
-
-	g_free (old_location);
-
-	vfs_result = gnome_vfs_xfer_uri (source_uri,
-					 dest_uri,
-					 GNOME_VFS_XFER_FOLLOW_LINKS,
-					 GNOME_VFS_XFER_ERROR_MODE_ABORT,
-					 GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-					 NULL, NULL);
-
-	gnome_vfs_uri_unref (source_uri);
-	gnome_vfs_uri_unref (dest_uri);
-
-	if (vfs_result != GNOME_VFS_OK) {
+	copied = g_file_copy (source, dest, G_FILE_COPY_OVERWRITE,
+			      NULL, NULL, NULL, NULL);
+	
+	if (!copied) {
 		g_free (new_location);
 		return FALSE;
 	}
@@ -1137,6 +1151,8 @@ void
 panel_launcher_set_dnd_enabled (Launcher *launcher,
 				gboolean  dnd_enabled)
 {
+	GdkPixbuf *pixbuf;
+
 	if (dnd_enabled) {
 		static GtkTargetEntry dnd_targets[] = {
 			{ "application/x-panel-icon-internal", 0, TARGET_ICON_INTERNAL },
@@ -1149,9 +1165,12 @@ panel_launcher_set_dnd_enabled (Launcher *launcher,
 				     dnd_targets, 2,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
 		//FIXME: this doesn't work since the pixbuf isn't loaded yet
-		if (BUTTON_WIDGET (launcher->button)->pixbuf)
+		pixbuf = button_widget_get_pixbuf (BUTTON_WIDGET (launcher->button));
+		if (pixbuf) {
 			gtk_drag_source_set_icon_pixbuf (launcher->button,
-							 BUTTON_WIDGET (launcher->button)->pixbuf);
+							 pixbuf);
+			g_object_unref (pixbuf);
+		}
 		GTK_WIDGET_SET_FLAGS (launcher->button, GTK_NO_WINDOW);
 	
 
